<?php
/**
 * @copyright: Copyright © 2020 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */

namespace Firebear\PlatformSap\Model;

use Firebear\PlatformSap\Api\LogRepositoryInterface;
use Firebear\PlatformSap\Api\Data\LogInterface;
use Firebear\PlatformSap\Model\LogFactory;
use Firebear\PlatformSap\Model\ResourceModel\Log as ObjectResourceModel;
use Firebear\PlatformSap\Model\ResourceModel\Log\CollectionFactory;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Api\SearchResultsInterfaceFactory;
use Magento\Framework\Api\SortOrder;

/**
 * Class LogRepository
 * @package Firebear\PlatformSap\Model
 */
class LogRepository implements LogRepositoryInterface
{
    /**
     * @var LogFactory
     */
    protected $objectFactory;
    /**
     * @var ObjectResourceModel
     */
    protected $objectResourceModel;
    /**
     * @var CollectionFactory
     */
    protected $collectionFactory;
    /**
     * @var SearchResultsInterfaceFactory
     */
    protected $searchResultsFactory;
    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;

    const STATUS_SUCCESS = 2;
    const STATUS_ERROR = 1;

    /**
     * LogRepository constructor.
     * @param \Firebear\PlatformSap\Model\LogFactory $objectFactory
     * @param ObjectResourceModel $objectResourceModel
     * @param CollectionFactory $collectionFactory
     * @param SearchResultsInterfaceFactory $searchResultsFactory
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     */
    public function __construct(
        LogFactory $objectFactory,
        ObjectResourceModel $objectResourceModel,
        CollectionFactory $collectionFactory,
        SearchResultsInterfaceFactory $searchResultsFactory,
        SearchCriteriaBuilder $searchCriteriaBuilder
    ) {
        $this->objectFactory        = $objectFactory;
        $this->objectResourceModel  = $objectResourceModel;
        $this->collectionFactory    = $collectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
    }


    /**
     * @param $data
     * @return LogInterface
     * @throws CouldNotSaveException
     */
    public function createByData($data)
    {
        $object = $this->objectFactory->create();
        $object->setEntity($data['entity']);
        $object->setEntityId($data['entity_id']);
        return $this->save($object);
    }

    /**
     * @param $object
     * @param $data
     * @throws CouldNotSaveException
     */
    public function updateByData(LogInterface $object, $data)
    {
        $object->setResult($data['message']);
        $object->setStatus($data['status']);
        $this->save($object);
        if ($data['status'] == self::STATUS_SUCCESS) {
            $logs = (array)($this->getList(
                $this->searchCriteriaBuilder
                    ->addFilter('entity_id', $object->getEntityId())
                    ->addFilter('entity', $object->getEntity())
                    ->addFilter('status', self::STATUS_ERROR)
                    ->create()
            )->getItems());
            if (!empty($logs)) {
                foreach ($logs as $log) {
                    $log->setStatus(self::STATUS_SUCCESS);
                    $this->save($log);
                }
            }
        }
    }

    /**
     * @param LogInterface $object
     * @return LogInterface
     * @throws CouldNotSaveException
     */
    public function save(LogInterface $object)
    {
        try {
            $this->objectResourceModel->save($object);
        } catch (\Exception $e) {
            throw new CouldNotSaveException(__($e->getMessage()));
        }
        return $object;
    }

    /**
     * @param $id
     * @return mixed
     * @throws NoSuchEntityException
     */
    public function getById($id)
    {
        $object = $this->objectFactory->create();
        $this->objectResourceModel->load($object, $id);
        if (!$object->getId()) {
            throw new NoSuchEntityException(__('Object with id "%1" does not exist.', $id));
        }
        return $object;
    }

    /**
     * @param LogInterface $object
     * @return bool
     * @throws CouldNotDeleteException
     */
    public function delete(LogInterface $object)
    {
        try {
            $this->objectResourceModel->delete($object);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__($exception->getMessage()));
        }
        return true;
    }

    /**
     * @param $id
     * @return bool
     * @throws CouldNotDeleteException
     * @throws NoSuchEntityException
     */
    public function deleteById($id)
    {
        return $this->delete($this->getById($id));
    }

    /**
     * @param SearchCriteriaInterface $criteria
     * @return mixed
     */
    public function getList(SearchCriteriaInterface $criteria)
    {
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);
        $collection = $this->collectionFactory->create();
        foreach ($criteria->getFilterGroups() as $filterGroup) {
            $fields = [];
            $conditions = [];
            foreach ($filterGroup->getFilters() as $filter) {
                $condition = $filter->getConditionType() ? $filter->getConditionType() : 'eq';
                $fields[] = $filter->getField();
                $conditions[] = [$condition => $filter->getValue()];
            }
            if ($fields) {
                $collection->addFieldToFilter($fields, $conditions);
            }
        }
        $searchResults->setTotalCount($collection->getSize());
        $sortOrders = $criteria->getSortOrders();
        if ($sortOrders) {
            /** @var SortOrder $sortOrder */
            foreach ($sortOrders as $sortOrder) {
                $collection->addOrder(
                    $sortOrder->getField(),
                    ($sortOrder->getDirection() == SortOrder::SORT_ASC) ? 'ASC' : 'DESC'
                );
            }
        }
        $collection->setCurPage($criteria->getCurrentPage());
        $collection->setPageSize($criteria->getPageSize());
        $objects = [];
        foreach ($collection as $objectModel) {
            $objects[] = $objectModel;
        }
        $searchResults->setItems($objects);
        return $searchResults;
    }
}
