<?php
/**
 * @copyright: Copyright © 2021 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */

namespace Firebear\PlatformSap\Model;

use Firebear\ImportExport\Model\Import\Product;
use Firebear\PlatformSap\Api\ProductRepositoryInterface;
use Magento\Catalog\Api\Data\ProductAttributeInterface;
use Magento\Catalog\Model\ProductRepository as BaseProductRepository;
use Magento\Eav\Api\AttributeOptionManagementInterface;
use Magento\Eav\Api\Data\AttributeOptionInterfaceFactory as AttributeOptionFactory;
use Magento\Eav\Model\Config as EavConfig;
use Magento\Catalog\Api\Data\ProductInterface;
use Firebear\ImportExport\Model\Import\Product\CategoryProcessor;
use Firebear\PlatformSap\Model\Import\Source\Category;

/**
 * ProductRepository model
 */
class ProductRepository implements ProductRepositoryInterface
{
    /**
     * @var BaseProductRepository
     */
    protected $productRepository;

    /**
     * @var EavConfig
     */
    protected $eavConfig;

    /**
     * @var AttributeOptionManagementInterface
     */
    protected $eavOptionManagement;

    /**
     * @var AttributeOptionFactory
     */
    protected $attributeOptionFactory;

    /**
     * @var CategoryProcessor
     */
    protected $categoryProcessor;

    /**
     * ProductRepository constructor.
     * @param BaseProductRepository $productRepository
     * @param EavConfig $eavConfig
     * @param AttributeOptionManagementInterface $eavOptionManagement
     * @param AttributeOptionFactory $attributeOptionFactory
     * @param CategoryProcessor $categoryProcessor
     */
    public function __construct(
        BaseProductRepository $productRepository,
        EavConfig $eavConfig,
        AttributeOptionManagementInterface $eavOptionManagement,
        AttributeOptionFactory $attributeOptionFactory,
        CategoryProcessor $categoryProcessor
    ) {
        $this->productRepository = $productRepository;
        $this->eavConfig = $eavConfig;
        $this->eavOptionManagement = $eavOptionManagement;
        $this->attributeOptionFactory = $attributeOptionFactory;
        $this->categoryProcessor = $categoryProcessor;
    }

    /**
     * @param ProductInterface $entity
     * @param array $dropdownAttributes
     * @return ProductInterface
     */
    public function save(ProductInterface $entity, array $dropdownAttributes)
    {
        $customAttributes = [];
        if (!empty($dropdownAttributes)) {
            foreach ($dropdownAttributes as $attribute) {
                $code = $attribute->getAttributeCode();
                $label = $attribute->getDefaultFrontendLabel();
                $options = $this->initAttributeOptions($code);
                if (!isset($options[$label])) {
                    $optionId = $this->createNewOption($code, $label);
                    if (!empty($optionId)) {
                        $customAttributes[$code] = $optionId;
                    }
                } else {
                    $customAttributes[$code] = $options[$label];
                }
            }
        }

        if (!empty($customAttributes)) {
            foreach ($customAttributes as $customAttributeCode => $customAttributeId) {
                $entity->setCustomAttribute($customAttributeCode, $customAttributeId);
            }
        }

        $entity->setStoreId(0);
        $entity = $this->setProductCategories($entity);

        return $this->productRepository->save($entity);
    }

    /**
     * @param $entity
     * @return mixed
     */
    protected function setProductCategories($entity)
    {
        $extensionAttributes = $entity->getExtensionAttributes();
        $sapCategoryPath = $extensionAttributes->getSapCategoryPath();
        if (!empty($sapCategoryPath)) {
            $categories = explode(',', $sapCategoryPath);
            $categories = $this->prepareProductCategories($categories, '/');

            $categoriesResult = '';
            foreach ($categories as $categoryKey => $value) {
                $categoriesResult .=
                    Category::DEFAULT_CATEGORY_NAME;
                $categoriesResult .= '/';
                $categoriesResult .= $categoryKey . ',';
            }

            $categoriesResult = substr(
                $categoriesResult,
                0,
                -1
            );

            if (!empty($categoriesResult)) {
                $ids = $this->categoryProcessor->getRowCategories(
                    [Product::COL_CATEGORY => $categoriesResult],
                    ','
                );
                if (!empty($ids)) {
                    $entity->setCustomAttribute('category_ids', $ids);
                }
            }
        }

        return $entity;
    }

    /**
     * @param $categories
     * @param $levelSeparator
     * @return array
     */
    protected function prepareProductCategories($categories, $levelSeparator)
    {
        $result = [];
        foreach ($categories as $category) {
            $categoryData = explode($levelSeparator, $category);
            $categoryPath = '';
            foreach ($categoryData as $key => $value) {
                if (!empty($categoryPath)) {
                    $categoryPath .= $levelSeparator . trim($value);
                } else {
                    $categoryPath .= trim($value);
                }
                $result[$categoryPath] = true;
            }
        }
        return $result;
    }

    /**
     * @param $attributeCode
     * @return array
     */
    public function initAttributeOptions($attributeCode)
    {
        $result = [];
        $attribute = $this->eavConfig->getAttribute('catalog_product', $attributeCode);
        $options = $attribute->getSource()->getAllOptions();
        foreach ($options as $option) {
            $result[$option['label']] = $option['value'];
        }
        return $result;
    }

    /**
     * @param $attributeCode
     * @param $label
     * @return mixed
     */
    public function createNewOption($attributeCode, $label)
    {
        $option = $this->attributeOptionFactory->create();
        $option->setLabel($label);
        return $this->eavOptionManagement->add(
            ProductAttributeInterface::ENTITY_TYPE_CODE,
            $attributeCode,
            $option
        );
    }
}
